# -*- coding: utf-8 -*-
"""Python package to support the research during
the annual Lehrexkursion at Universität Hamburg.
"""
from . import csv
from . import math
from . import plots


__all__ = [
    'csv',
    'math',
    'plots',
]

__version__ = '0.5'
